/*
* ============================================================================
*  Name        : loopbackaudiocontrol.h
*  Part of     : mccexample / mccexengine
*  Description : CLoopbackAudioControl declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CLOOPBACKAUDIOCONTROL_H
#define C_CLOOPBACKAUDIOCONTROL_H

#include <e32base.h>

#include "maudiocontrol.h"
#include "tnetworksettings.h"

class CMccInterface;
class CMccEventHandler;
class CMccNetworkSession;
class CAudioStreamBase;
class CStreamStateMachine;
class TCodecSettings;

/**
 * CLoopbackAudioControl provides MAudioControl interface implementation.
 * 
 * It contains up- and downlink streams which transfer audio from audio input
 * to audio output (from mic to loudspeaker/earpiece) using the whole 
 * multimedia comms controller subsystem.
 */
NONSHARABLE_CLASS( CLoopbackAudioControl ) : public CBase,
    public MAudioControl
    {
public:
    
    /**
     * Destructor.
     */
    virtual ~CLoopbackAudioControl();
    
    /**
     * Two-phased constructor.
     * 
     * @param aMcc Reference to MCC.
     * @param aEventHandler Reference to event handler.
     * @param aObserver Reference to the engine observer.
     * @param aSettings Network settings.
     * @param aCodec Codec settings.
     */
    static CLoopbackAudioControl* NewL( CMccInterface& aMcc,
            CMccEventHandler& aEventHandler,
            MMccExEngineObserver& aObserver,
            const TNetworkSettings& aSettings,
            const TCodecSettings& aCodec );
    
    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     * 
     * @param aMcc Reference to MCC.
     * @param aEventHandler Reference to event handler.
     * @param aObserver Reference to the engine observer.
     * @param aSettings Network settings.
     * @param aCodec Codec settings.
     */
    static CLoopbackAudioControl* NewLC( CMccInterface& aMcc,
            CMccEventHandler& aEventHandler,
            MMccExEngineObserver& aObserver,
            const TNetworkSettings& aSettings,
            const TCodecSettings& aCodec );
    
// from MAudioControl
    
    /**
     * From MAudioControl.
     * Prepares this audio control.
     * 
     * @see maudiocontrol.h
     */
    void PrepareAudioL();
    
    /**
     * From MAudioControl.
     * Starts this audio control.
     * 
     * @see maudiocontrol.h
     */
    void StartAudioL();
    
    /**
     * From MAudioControl.
     * Pauses this audio control.
     * 
     * @see maudiocontrol.h
     */
    void PauseAudioL();
    
    /**
     * From MAudioControl.
     * Resumes this paused audio control.
     * 
     * @see maudiocontrol.h
     */
    void ResumeAudioL();
    
    /**
     * From MAudioControl.
     * Stops/resets this audio control.
     * 
     * @see maudiocontrol.h
     */
    void StopAudioL();
    
    /**
     * From MAudioControl.
     * Sets the current audio output volume.
     * 
     * @see maudiocontrol.h
     */
    void SetVolumeL( TInt aVolume );
    
    /**
     * From MAudioControl.
     * Returns the current audio output volume.
     * 
     * @see maudiocontrol.h
     */
    TInt VolumeL();
    
    /**
     * From MAudioControl.
     * Returns the maximum supported volume level of the audio output.
     * 
     * @see maudiocontrol.h
     */
    TInt MaxVolumeL();
    
    /**
     * From MAudioControl.
     * Enables or disables mic mute.
     * 
     * @see maudiocontrol.h
     */
    void SetMuteStateL( TBool aMuteState );
    
    /**
     * From MAudioControl.
     * Sets the current audio routing (public or private).
     * 
     * @see maudiocontrol.h
     */
    void SetAudioRoutingL( TUint32 aRouting );
    
    /**
     * From MAudioControl.
     * Returns the current audio routing of local audio-output.
     * 
     * @see maudiocontrol.h
     */
    TUint32 AudioRoutingL();
    
    /**
     * From MAudioControl.
     * Returns the current state of this audio control.
     * 
     * @see tcontrolstate.h
     */
    TControlState State();
    
    /**
     * From MAudioControl.
     * Deletes this audio control.
     * 
     * @see maudiocontrol.h
     */
    void ReleaseAudioControl();

private:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aMcc Reference to MCC.
     * @param aEventHandler Reference to event handler.
     * @param aSettings Network settings.
     */
    CLoopbackAudioControl( CMccInterface& aMcc,
            CMccEventHandler& aEventHandler,
            const TNetworkSettings& aSettings );
    
    /**
     * Symbian default constructor for performing 2nd stage construction.
     * 
     * @param aObserver Reference to the engine observer.
     * @param aCodec Codec settings.
     */
    void ConstructL( MMccExEngineObserver& aObserver,
        const TCodecSettings& aCodec );
    
    /**
     * Finds a codec by its name from the given capability array.
     * 
     * @param aName Codec name to search.
     * @param aCapabilities Capability array.
     * @leave KErrNotFound If no codec is found.
     * @return Pointer to the found codec.
     */
    CMccCodecInformation* FindCodecByNameL( const TDesC8& aName, 
        const RPointerArray<CMccCodecInformation>& aCapabilities );
    
private:    // Data
    
    /**
     * Reference to MCC.
     */
    CMccInterface& iMcc;
    
    /**
     * Reference to event handler.
     */
    CMccEventHandler& iEventHandler;
    
    /**
     * Network session.
     * Own
     */
    CMccNetworkSession* iSession;
    
    /**
     * State machine for streams.
     * Own
     */
    CStreamStateMachine* iStateMachine;
    
    /**
     * Array of audio streams.
     * Own
     */
    RPointerArray<CAudioStreamBase> iAudioStreams;
    
    /**
     * Network settings.
     */
    TNetworkSettings iNwSettings;
    
    };

#endif  // C_CLOOPBACKAUDIOCONTROL_H
