/*
* ============================================================================
*  Name        : loopbackvideocontrol.cpp
*  Part of     : mccexample / mccexengine
*  Description : CLoopbackVideoControl class implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include <mmccinterface.h>
#include <mmcccodecinformation.h>

#include "loopbackvideocontrol.h"
#include "mccnetworksession.h"
#include "mcceventhandler.h"
#include "uplinkvideostream.h"
#include "downlinkvideostream.h"
#include "streamstatemachine.h"
#include "tvideodeviceparameters.h"
#include "mccexpanic.pan"


// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CLoopbackVideoControl::CLoopbackVideoControl
// ---------------------------------------------------------------------------
//
CLoopbackVideoControl::CLoopbackVideoControl( CMccInterface& aMcc,
    CMccEventHandler& aEventHandler, const TNetworkSettings& aSettings ) : 
    iMcc( aMcc ), 
    iEventHandler( aEventHandler ),
    iNwSettings( aSettings )
    {
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::~CLoopbackVideoControl
// ---------------------------------------------------------------------------
//
CLoopbackVideoControl::~CLoopbackVideoControl()
    {
    iEventHandler.RemoveSession( iSession );
    iVideoStreams.ResetAndDestroy();
    delete iSession;
    delete iStateMachine;
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::NewLC
// ---------------------------------------------------------------------------
//
CLoopbackVideoControl* CLoopbackVideoControl::NewLC( CMccInterface& aMcc,
    CMccEventHandler& aEventHandler,
    MMccExEngineObserver& aObserver,
    const TRect& aRect,
    const TSize& aResolution,
    const TVideoDeviceParameters& aParams,
    const TNetworkSettings& aSettings )
    {
    CLoopbackVideoControl* self = 
        new ( ELeave ) CLoopbackVideoControl(
            aMcc, aEventHandler, aSettings );
    
    CleanupStack::PushL( self );
    self->ConstructL( aObserver,
            aRect,
            aResolution,
            aParams );
    
    return self;
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::NewL
// ---------------------------------------------------------------------------
//
CLoopbackVideoControl* CLoopbackVideoControl::NewL( CMccInterface& aMcc,
    CMccEventHandler& aEventHandler,
    MMccExEngineObserver& aObserver,
    const TRect& aRect,
    const TSize& aResolution,
    const TVideoDeviceParameters& aParams,
    const TNetworkSettings& aSettings )
    {
    CLoopbackVideoControl* self = 
        CLoopbackVideoControl::NewLC( aMcc,
                aEventHandler,
                aObserver,
                aRect,
                aResolution,
                aParams,
                aSettings);
    
    CleanupStack::Pop( self );
    return self;
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::ConstructL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::ConstructL( MMccExEngineObserver& aObserver,
    const TRect& aRect,
    const TSize& aResolution,
    const TVideoDeviceParameters& aParams )
    {
    iStateMachine = CStreamStateMachine::NewL();
    iSession = CMccNetworkSession::NewL( iMcc, aObserver,
        iNwSettings.iMediaQoS, iNwSettings.iRtpPort );
    
    iEventHandler.AddSessionL( iSession );
    
    RPointerArray<CMccCodecInformation> codecs;
    CleanupResetAndDestroyPushL( codecs );
    User::LeaveIfError( iMcc.GetCapabilities( codecs ) );
    
    // The linkID at this point is actually correct, only thing is that the
    // link is not up until MCC sends 'Link created' - event. Thus we can
    // safely start creating the streams at this point. Also, the streams do 
    // not reserve any MCC resources at this point. That must be done 
    // separately and later once the link creation is finished.
    CMccCodecInformation* codec = FindCodecL( KUidMediaTypeVideo, codecs );
    CVideoStreamBase* stream( NULL );
    
    stream = CUplinkVideoStream::NewLC( *iStateMachine, *codec,
            iMcc,
            iSession->SessionId(), iSession->LinkId(),
            aResolution,
            aParams.iCameraIndex );
    
    stream->SetRtcpUsage( iNwSettings.iRtcpEnabled );
    
    iVideoStreams.AppendL( stream );
    iSession->AddStreamL( *stream );
    CleanupStack::Pop( stream );
    stream = NULL;
    
    stream = CDownlinkVideoStream::NewLC( *iStateMachine, *codec, 
            iMcc,
            iSession->SessionId(), iSession->LinkId(), 
            aRect,
            aParams.iWindowOrdinalPosition,
            aParams.iWindowOrdinalPriority,
            aParams.iScreenIndex );
    
    stream->SetRtcpUsage( iNwSettings.iRtcpEnabled );
    
    iVideoStreams.AppendL( stream );
    iSession->AddStreamL( *stream );
    CleanupStack::Pop( stream );
    stream = NULL;
    
    CleanupStack::PopAndDestroy( &codecs );
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::PrepareVideoL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::PrepareVideoL()
    {
    TInetAddr address( iNwSettings.iRemoteAddress );
    address.SetPort( iNwSettings.iRtpPort );
    
    iSession->SetRemoteAddressL( address );
    
    TInt count( iVideoStreams.Count() );
    while ( count )
        {
        count -= 1;
        iVideoStreams[count]->PrepareStreamL();
        }
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::StartVideoL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::StartVideoL()
    {
    TInt count( iVideoStreams.Count() );
    while ( count )
        {
        count -= 1;
        iVideoStreams[count]->StartStreamL();
        }
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::PauseVideoL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::PauseVideoL()
    {
    const TInt count( iVideoStreams.Count() );
    TInt k = 0;
    
    // First pause the uplink streams so the downlink goes into standby state
    // and after that, pause it explicitly
    for ( k = 0; k < count; k++ )
        {
        if ( KMccVideoUplinkStream == iVideoStreams[k]->StreamType() )
            {
            iVideoStreams[k]->PauseStreamL();
            }
        }
    
    for ( k = 0; k < count; k++ )
        {
        if ( KMccVideoDownlinkStream == iVideoStreams[k]->StreamType() )
            {
            iVideoStreams[k]->PauseStreamL();
            }
        }
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::ResumeVideoL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::ResumeVideoL()
    {
    TInt count( iVideoStreams.Count() );
    while ( count )
        {
        count -= 1;
        iVideoStreams[count]->ResumeStreamL();
        }
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::StopVideoL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::StopVideoL()
    {
    const TInt count( iVideoStreams.Count() );
    TInt k = 0;
    
    // First stop the uplink streams so the downlink goes into standby state
    // and after that, stop it explicitly
    for ( k = 0; k < count; k++ )
        {
        if ( KMccVideoUplinkStream == iVideoStreams[k]->StreamType() )
            {
            iVideoStreams[k]->StopStreamL();
            }
        }
    
    for ( k = 0; k < count; k++ )
        {
        if ( KMccVideoDownlinkStream == iVideoStreams[k]->StreamType() )
            {
            iVideoStreams[k]->StopStreamL();
            }
        }
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::ReleaseVideoControl
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::ReleaseVideoControl()
    {
    delete this;
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::State
// ---------------------------------------------------------------------------
//
TControlState CLoopbackVideoControl::State()
    {
    const TInt count( iVideoStreams.Count() );
    if ( count )
        {
        TControlState state ( iVideoStreams[0]->State() );
        for ( TInt k = 1; k < count; k++ )
            {
            if ( state != iVideoStreams[k]->State() )
                {
                return EStateChanging;
                }
            }
        
        return state;
        }
    else
        {
        MccExPanic( EMccExUnknownState );
        return EStateChanging;
        }
    }


// ---------------------------------------------------------------------------
// CLoopbackVideoControl::UpdateScreenRectL
// ---------------------------------------------------------------------------
//
void CLoopbackVideoControl::UpdateScreenRectL( const TRect& aRect,
    TInt aWindowOrdinalPosition,
    TInt aWindowOrdinalPriority,
    TInt aScreenRotation )
    {
    TInt idx = iVideoStreams.Count();
    while( idx )
        {
        idx -= 1;
        if ( KMccVideoDownlinkStream == iVideoStreams[idx]->StreamType() )
            {
            static_cast<CDownlinkVideoStream*>(
                    iVideoStreams[idx] )->UpdateScreenRectL( aRect,
                            aWindowOrdinalPosition,
                            aWindowOrdinalPriority,
                            aScreenRotation );
            }
        }
    }

// ---------------------------------------------------------------------------
// CLoopbackVideoControl::FindCodecL
// ---------------------------------------------------------------------------
//
CMccCodecInformation* CLoopbackVideoControl::FindCodecL( const TUid& aType,
    const RPointerArray<CMccCodecInformation>& aCodecs )
    {
    CMccCodecInformation* ret( NULL );
    TInt idx( aCodecs.Count() );
    while( idx || !ret )
        {
        idx -= 1;
        if ( aType == aCodecs[idx]->Type() )
            {
            ret = aCodecs[idx];
            }
        }
    
    if ( !ret )
        {
        User::Leave( KErrNotFound );
        }
    
    return ret;
    }
