/*
* ============================================================================
*  Name        : streambase.cpp
*  Part of     : mccexample / mccexengine
*  Description : CStreamBase class implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include <mmccinterface.h>
#include <mmcccodecinformation.h>

#include "streambase.h"
#include "streamstatemachine.h"
#include "streamstatebase.h"
#include "mccexpanic.pan"

// ======== LOCAL FUNCTIONS ========

// ---------------------------------------------------------------------------
// Basic assertion function for statechanging functions
// ---------------------------------------------------------------------------
//
static void StateChangeAssertionL( TBool aLinkReady, TUint32 aStreamId,
    CStreamStateBase* aState )
    {
    // 1. Link needs to be ready
    // 2. Stream ID needs to be other than KMaxTUint32
    // 3. State must be NULL (state change not ongoing)
    __ASSERT_ALWAYS( aLinkReady && !aState, User::Leave( KErrNotReady ) );
    __ASSERT_ALWAYS( KMaxTUint32 != aStreamId, User::Leave( KErrNotReady ) );
    }


// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CStreamBase::CStreamBase
// ---------------------------------------------------------------------------
//
CStreamBase::CStreamBase( CStreamStateMachine& aStateMachine, 
    CMccInterface& aMcc, TUint32 aSession, TUint32 aLink ) : 
    iControlState( EStateChanging ), 
    iStateMachine( aStateMachine ), 
    iMcc( aMcc ),
    iSessionId( aSession ), 
    iLinkId( aLink ), 
    iStreamId( KMaxTUint32 ),
    iSourceEndpoint( KMaxTUint32 ), 
    iSinkEndpoint( KMaxTUint32 ),
    iRtcpUsage( ETrue )
    {
    }


// ---------------------------------------------------------------------------
// CStreamBase::~CStreamBase
// ---------------------------------------------------------------------------
//
CStreamBase::~CStreamBase()
    {
    if ( KMaxTUint32 != iStreamId )
        {
        iMcc.DeleteStream( iSessionId, iLinkId, iStreamId );
        }
    
    delete iCodec;
    
    // Not owned objects NULL'ed explicitly
    iState = NULL;
    }


// ---------------------------------------------------------------------------
// CStreamBase::SetRtcpUsage
// ---------------------------------------------------------------------------
//
void CStreamBase::SetRtcpUsage( TBool aRtcpUsage )
    {
    iRtcpUsage = aRtcpUsage;
    }


// ---------------------------------------------------------------------------
// CStreamBase::RtcpUsage
// ---------------------------------------------------------------------------
//
TBool CStreamBase::RtcpUsage() const
    {
    return iRtcpUsage;
    }


// ---------------------------------------------------------------------------
// CStreamBase::StreamType
// ---------------------------------------------------------------------------
//
TInt CStreamBase::StreamType() const
    {
    return iStreamType;
    }


// ---------------------------------------------------------------------------
// CStreamBase::LinkReady
// ---------------------------------------------------------------------------
//
TInt CStreamBase::LinkReady()
    {
    TRAPD( error, CreateStreamL() );
    if ( KErrNone == error )
        {
        iLinkReady = ETrue;
        iControlState = EStateStopped;
        }
    
    return error;
    }


// ---------------------------------------------------------------------------
// CStreamBase::SessionId
// ---------------------------------------------------------------------------
//
TUint32 CStreamBase::SessionId() const
    {
    return iSessionId;
    }


// ---------------------------------------------------------------------------
// CStreamBase::LinkId
// ---------------------------------------------------------------------------
//
TUint32 CStreamBase::LinkId() const
    {
    return iLinkId;
    }


// ---------------------------------------------------------------------------
// CStreamBase::StreamId
// ---------------------------------------------------------------------------
//
TUint32 CStreamBase::StreamId() const
    {
    return iStreamId;
    }


// ---------------------------------------------------------------------------
// CStreamBase::SinkEndpointId
// ---------------------------------------------------------------------------
//
TUint32 CStreamBase::SinkEndpointId() const
    {
    return iSinkEndpoint;
    }


// ---------------------------------------------------------------------------
// CStreamBase::SourceEndpointId
// ---------------------------------------------------------------------------
//
TUint32 CStreamBase::SourceEndpointId() const
    {
    return iSourceEndpoint;
    }


// ---------------------------------------------------------------------------
// CStreamBase::SinkIsReady
// ---------------------------------------------------------------------------
//
void CStreamBase::SinkIsReady()
    {
    iSinkReady = ETrue;
    }


// ---------------------------------------------------------------------------
// CStreamBase::SourceIsReady
// ---------------------------------------------------------------------------
//
void CStreamBase::SourceIsReady()
    {
    iSourceReady = ETrue;
    }


// ---------------------------------------------------------------------------
// CStreamBase::PrepareStreamL
// ---------------------------------------------------------------------------
//
void CStreamBase::PrepareStreamL()
    {
    StateChangeAssertionL( iLinkReady, iStreamId, iState );
    
    CStreamStateBase& state(
            iStateMachine.RequestStateL( EStatePrepared ) );
    state.EnterStateL( *this, iStateMachine.RequestStateL( iControlState ) );
    
    iState = &state;
    iControlState = EStateChanging;
    }


// ---------------------------------------------------------------------------
// CStreamBase::PrepareStreamL
// ---------------------------------------------------------------------------
//
void CStreamBase::StartStreamL()
    {
    StateChangeAssertionL( iLinkReady, iStreamId, iState );
    
    CStreamStateBase& state(
            iStateMachine.RequestStateL( EStateStarted ) );
    state.EnterStateL( *this, iStateMachine.RequestStateL( iControlState ) );
    
    iState = &state;
    iControlState = EStateChanging;
    }


// ---------------------------------------------------------------------------
// CStreamBase::PauseStreamL
// ---------------------------------------------------------------------------
//
void CStreamBase::PauseStreamL()
    {
    StateChangeAssertionL( iLinkReady, iStreamId, iState );
    
    CStreamStateBase& state( 
            iStateMachine.RequestStateL( EStatePaused ) );
    state.EnterStateL( *this, iStateMachine.RequestStateL( iControlState ) );
    
    iState = &state;
    iControlState = EStateChanging;
    }


// ---------------------------------------------------------------------------
// CStreamBase::ResumeStreamL
// ---------------------------------------------------------------------------
//
void CStreamBase::ResumeStreamL()
    {
    StateChangeAssertionL( iLinkReady, iStreamId, iState );
    
    CStreamStateBase& state(
            iStateMachine.RequestStateL( EStateStarted ) );
    state.EnterStateL( *this, iStateMachine.RequestStateL( iControlState ) );
    
    iState = &state;
    iControlState = EStateChanging;
    }


// ---------------------------------------------------------------------------
// CStreamBase::StopStreamL
// ---------------------------------------------------------------------------
//
void CStreamBase::StopStreamL()
    {
    StateChangeAssertionL( iLinkReady, iStreamId, iState );
    
    CStreamStateBase& state( 
            iStateMachine.RequestStateL( EStateStopped ) );
    state.EnterStateL( *this, iStateMachine.RequestStateL( iControlState ) );
    
    iControlState = EStateStopped;
    
    // Reset the endpoint ID's so that the creation succeeds next time if
    // needed.
    iSinkEndpoint = KMaxTUint32;
    iSourceEndpoint = KMaxTUint32;
    }


// ---------------------------------------------------------------------------
// CStreamBase::State
// ---------------------------------------------------------------------------
//
TControlState CStreamBase::State() const
    {
    return iControlState;
    }


// ---------------------------------------------------------------------------
// CStreamBase::MccInterface
// ---------------------------------------------------------------------------
//
CMccInterface& CStreamBase::MccInterface()
    {
    return iMcc;
    }


// ---------------------------------------------------------------------------
// CStreamBase::Codec
// ---------------------------------------------------------------------------
//
CMccCodecInformation& CStreamBase::Codec()
    {
    // Assert that we have the codec - otherwise object is not properly
    // initialized.
    __ASSERT_ALWAYS( iCodec, MccExPanic( EMccExNullPtr ) );
    return *iCodec;
    }


// ---------------------------------------------------------------------------
// CStreamBase::HandleEvent
// ---------------------------------------------------------------------------
//
void CStreamBase::HandleEvent( const TMccEvent& aEvent )
    {
    RDebug::Print( _L( "!- CStreamBase::HandleEvent, entry -!" ) );
    
    if ( aEvent.iSessionId != iSessionId ||
         aEvent.iLinkId != iLinkId || 
         aEvent.iStreamId != iStreamId || 
         KMccEventCategoryStream != aEvent.iEventCategory )
        {
        MccExPanic( EMccExEventCategory );
        }
    else if ( iState )
        {
        iState->HandleMccEvent( aEvent, *this );
        }
    else if ( EStateChanging != iControlState )
        {
        TRAP_IGNORE( iStateMachine.RequestStateL(
                iControlState ).HandleMccEvent( aEvent, *this ) )
        }
    // else a no-operation
    
    // If the state change is ready for sink and source, reset the flags and
    // store audiocontrol state.
    if ( iSinkReady && iSourceReady )
        {
        iControlState = iState->ControlState();
        iState = NULL;
        iSinkReady = EFalse;
        iSourceReady = EFalse;
        }
    
    RDebug::Print( _L( "!- CStreamBase::HandleEvent, exit -!" ) );
    }


// ---------------------------------------------------------------------------
// CStreamBase::ConstructL
// ---------------------------------------------------------------------------
//
void CStreamBase::ConstructL( CMccCodecInformation& aCodec, TInt aType )
    {
    // Assert that we have not yet been initialized and the stream type falls
    // into proper range.
    __ASSERT_ALWAYS( !iCodec, User::Leave( KErrAlreadyExists ) );
    __ASSERT_ALWAYS( Rng( KMccAudioUplinkStream, aType, KMccDtmfStream ),
            User::Leave( KErrArgument ) );
    
    iCodec = aCodec.CloneDetailedL();
    iStreamType = aType;
    }


// ---------------------------------------------------------------------------
// CStreamBase::AddSourceL
// ---------------------------------------------------------------------------
//
void CStreamBase::AddSourceL( const TUid& aType, const TDesC8& aInitData )
    {
    // Assert that source has not yet been added.
    __ASSERT_ALWAYS( KMaxTUint32 == iSourceEndpoint,
            User::Leave( KErrAlreadyExists ) );
    
    User::LeaveIfError(
            iMcc.AddDataSource( aType, aInitData, iSourceEndpoint ) );
    }


// ---------------------------------------------------------------------------
// CStreamBase::AddSinkL
// ---------------------------------------------------------------------------
//
void CStreamBase::AddSinkL( const TUid& aType, const TDesC8& aInitData )
    {
    // Assert that sink has not yet been added.
    __ASSERT_ALWAYS( KMaxTUint32 == iSinkEndpoint,
               User::Leave( KErrAlreadyExists ) );
    
    User::LeaveIfError(
            iMcc.AddDataSink( aType, aInitData, iSinkEndpoint ) );
    }


// ---------------------------------------------------------------------------
// CStreamBase::CreateStreamL
// ---------------------------------------------------------------------------
//
void CStreamBase::CreateStreamL()
    {
    // Assert that internal state is proper for creating the streams. 
    __ASSERT_ALWAYS( iCodec && !iState && KMaxTUint32 == iStreamId,
            User::Leave( KErrNotReady ) );
    __ASSERT_ALWAYS( KMaxTUint32 != iSinkEndpoint &&
                     KMaxTUint32 != iSourceEndpoint,
            User::Leave( KErrNotReady ) );
    
    User::LeaveIfError( iMcc.CreateStream(
                        iSessionId,
                        iLinkId,
                        iStreamId,
                        iStreamType,
                        *iCodec ) );
    }
